@@ -28,6 +28,7 @@
 #include "pango-font.h"
 #include "pango-enum-types.h"
 #include "pango-impl-utils.h"
+#include "pango-utils-internal.h"
 
 /* FIXME */
 #define _(x) x
@@ -952,7 +953,7 @@ span_parse_int (const char *attr_name,
 {
   const char *end = attr_val;
 
-  if (!pango_scan_int (&end, val) || *end != '\0')
+  if (!_pango_scan_int (&end, val) || *end != '\0')
     {
       g_set_error (error,
 		   G_MARKUP_ERROR,
@@ -999,14 +1000,20 @@ span_parse_boolean (const char *attr_name,
   return TRUE;
 }
 
+extern gboolean
+_pango_color_parse_with_alpha (PangoColor *color,
+                               guint16    *alpha,
+                               const char *spec);
+
 static gboolean
 span_parse_color (const char *attr_name,
 		  const char *attr_val,
 		  PangoColor *color,
+                  guint16 *alpha,
 		  int line_number,
 		  GError **error)
 {
-  if (!pango_color_parse (color, attr_val))
+  if (!_pango_color_parse_with_alpha (color, alpha, attr_val))
     {
       g_set_error (error,
 		   G_MARKUP_ERROR,
@@ -1021,6 +1028,56 @@ span_parse_color (const char *attr_name,
   return TRUE;
 }
 
+static gboolean
+span_parse_alpha (const char  *attr_name,
+                  const char  *attr_val,
+                  guint16     *val,
+                  int          line_number,
+                  GError     **error)
+{
+  const char *end = attr_val;
+  int int_val;
+
+  if (_pango_scan_int (&end, &int_val))
+    {
+      if (*end == '\0' && int_val > 0 && int_val <= 0xffff)
+        {
+          *val = (guint16)int_val;
+          return TRUE;
+        }
+      else if (*end == '%' && int_val > 0 && int_val <= 100)
+        {
+          *val = (guint16)(int_val * 0xffff / 100);
+          return TRUE;
+        }
+      else
+        {
+          g_set_error (error,
+                       G_MARKUP_ERROR,
+                       G_MARKUP_ERROR_INVALID_CONTENT,
+                       _("Value of '%s' attribute on <span> tag "
+                         "on line %d could not be parsed; "
+                         "should be between 0 and 65536 or a "
+                         "percentage, not '%s'"),
+                         attr_name, line_number, attr_val);
+          return FALSE;
+        }
+    }
+  else
+    {
+      g_set_error (error,
+		   G_MARKUP_ERROR,
+		   G_MARKUP_ERROR_INVALID_CONTENT,
+		   _("Value of '%s' attribute on <span> tag "
+		     "on line %d could not be parsed; "
+		     "should be an integer, not '%s'"),
+		   attr_name, line_number, attr_val);
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
 static gboolean
 span_parse_enum (const char *attr_name,
 		 const char *attr_val,
@@ -1031,7 +1088,7 @@ span_parse_enum (const char *attr_name,
 {
   char *possible_values = NULL;
 
-  if (!pango_parse_enum (type, attr_val, val, FALSE, &possible_values))
+  if (!_pango_parse_enum (type, attr_val, val, FALSE, &possible_values))
     {
       g_set_error (error,
 		   G_MARKUP_ERROR,
@@ -1078,6 +1135,8 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
   const char *gravity = NULL;
   const char *gravity_hint = NULL;
   const char *font_features = NULL;
+  const char *alpha = NULL;
+  const char *background_alpha = NULL;
 
   g_markup_parse_context_get_position (context,
 				       &line_number, &char_number);
@@ -1106,6 +1165,18 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
       gboolean found = FALSE;
 
       switch (names[i][0]) {
+      case 'a':
+        CHECK_ATTRIBUTE (alpha);
+        break;
+      case 'b':
+	CHECK_ATTRIBUTE (background);
+	CHECK_ATTRIBUTE2(background, "bgcolor");
+        CHECK_ATTRIBUTE (background_alpha);
+        CHECK_ATTRIBUTE2(background_alpha, "bgalpha");
+        break;
+      case 'c':
+	CHECK_ATTRIBUTE2(foreground, "color");
+        break;
       case 'f':
 	CHECK_ATTRIBUTE (fallback);
 	CHECK_ATTRIBUTE2(desc, "font");
@@ -1121,6 +1192,7 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 
 	CHECK_ATTRIBUTE (foreground);
 	CHECK_ATTRIBUTE2(foreground, "fgcolor");
+	CHECK_ATTRIBUTE2(alpha, "fgalpha");
 
 	CHECK_ATTRIBUTE (font_features);
 	break;
@@ -1144,9 +1216,6 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 	CHECK_ATTRIBUTE (underline_color);
 	break;
       default:
-	CHECK_ATTRIBUTE (background);
-	CHECK_ATTRIBUTE2(background, "bgcolor");
-	CHECK_ATTRIBUTE2(foreground, "color");
 	CHECK_ATTRIBUTE (rise);
 	CHECK_ATTRIBUTE (variant);
 	CHECK_ATTRIBUTE (weight);
@@ -1193,7 +1262,7 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 	  const char *end;
 	  gint n;
 
-	  if ((end = size, !pango_scan_int (&end, &n)) || *end != '\0' || n < 0)
+	  if ((end = size, !_pango_scan_int (&end, &n)) || *end != '\0' || n < 0)
 	    {
 	      g_set_error (error,
 			   G_MARKUP_ERROR,
@@ -1321,21 +1390,47 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
   if (G_UNLIKELY (foreground))
     {
       PangoColor color;
+      guint16 alpha;
 
-      if (!span_parse_color ("foreground", foreground, &color, line_number, error))
+      if (!span_parse_color ("foreground", foreground, &color, &alpha, line_number, error))
 	goto error;
 
       add_attribute (tag, pango_attr_foreground_new (color.red, color.green, color.blue));
+      if (alpha != 0)
+        add_attribute (tag, pango_attr_foreground_alpha_new (alpha));
     }
 
   if (G_UNLIKELY (background))
     {
       PangoColor color;
+      guint16 alpha;
 
-      if (!span_parse_color ("background", background, &color, line_number, error))
+      if (!span_parse_color ("background", background, &color, &alpha, line_number, error))
 	goto error;
 
       add_attribute (tag, pango_attr_background_new (color.red, color.green, color.blue));
+      if (alpha != 0)
+        add_attribute (tag, pango_attr_background_alpha_new (alpha));
+    }
+
+  if (G_UNLIKELY (alpha))
+    {
+      guint16 val;
+
+      if (!span_parse_alpha ("alpha", alpha, &val, line_number, error))
+        goto error;
+
+      add_attribute (tag, pango_attr_foreground_alpha_new (val));
+    }
+
+  if (G_UNLIKELY (background_alpha))
+    {
+      guint16 val;
+
+      if (!span_parse_alpha ("background_alpha", background_alpha, &val, line_number, error))
+        goto error;
+
+      add_attribute (tag, pango_attr_background_alpha_new (val));
     }
 
   if (G_UNLIKELY (underline))
@@ -1352,7 +1447,7 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
     {
       PangoColor color;
 
-      if (!span_parse_color ("underline_color", underline_color, &color, line_number, error))
+      if (!span_parse_color ("underline_color", underline_color, &color, NULL, line_number, error))
 	goto error;
 
       add_attribute (tag, pango_attr_underline_color_new (color.red, color.green, color.blue));
@@ -1392,7 +1487,7 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
     {
       PangoColor color;
 
-      if (!span_parse_color ("strikethrough_color", strikethrough_color, &color, line_number, error))
+      if (!span_parse_color ("strikethrough_color", strikethrough_color, &color, NULL, line_number, error))
 	goto error;
 
       add_attribute (tag, pango_attr_strikethrough_color_new (color.red, color.green, color.blue));
